--  binding to libao (see xiph.org)
--  license is GPL or BSD, choose whatever you want
--  explicit permission goes to Debian to include this package under
--  the GPL
--  Copyright (C) 2005  Adrian Knoth <adi@thur.de>

--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.

--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.

--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--
--  rev 0.0.1      2005-03-12    Adrian Knoth    initial revision

with Interfaces.C; use Interfaces.C;
with Interfaces.C.Extensions; use Interfaces.C.Extensions;
with Interfaces.C.Strings; use Interfaces.C.Strings;
with Interfaces.C_Streams;

package ao is

   --  Ada-related types
   subtype driver is int;

   type sample is new Integer range -2**15 .. 2**15-1;
   for sample'Size use 16;

   type stereo_sample is record
       left, right : sample;
   end record;
   for stereo_sample'Size use 2*sample'Size;

   type sample_buffer is array (Positive range <>) of aliased sample;
   type stereo_buffer is array (Positive range <>) of aliased stereo_sample;
   type sample_buffer_ptr is access all sample_buffer;
   type stereo_buffer_ptr is access all stereo_buffer;


   --  constants imported from ao.h

   type AO_Type is (AO_Type_Live, AO_Type_File);
   for AO_Type use (AO_Type_Live => 1, AO_Type_File => 2);
   for AO_Type'Size use int'Size;

   type AO_Error is (AO_ENoDriver, AO_ENotFile, AO_ENotLive,
                     AO_EBadOption, AO_EOpenDevice, AO_OpenFile,
                     AO_EFileExists, AO_EFail);

   for AO_Error use (AO_ENoDriver   => 1,
                     AO_ENotFile    => 2,
                     AO_ENotLive    => 3,
                     AO_EBadOption  => 4,
                     AO_EOpenDevice => 5,
                     AO_OpenFile    => 6,
                     AO_EFileExists => 7,
                     AO_EFail       => 100);

   type AO_Fmt is (AO_Fmt_Little, AO_Fmt_Big, AO_Fmt_Native);
   for AO_Fmt use (AO_Fmt_Little => 1, AO_Fmt_Big => 2, AO_Fmt_Native => 4);
   for AO_Fmt'Size use int'Size;  --  AO_Fmt is int in C

   --  structures. Make sure binary representation fits to
   --  libao-C-compiler

   type ao_info is record
      ao_type : int;                   --  live output or file output?
      name : chars_ptr;                 --  full name of driver
      short_name : chars_ptr;           --  short name of driver
      author : chars_ptr;               --  driver author
      comment : chars_ptr;               --  driver comment
      preferred_byte_format : int;
      priority : int;
      options : char_array_access;
      option_count : int;
   end record;

   type ao_info_ptr is access ao_info;


   --  fixme: I don't understand anything...
   --  function access?
   type ao_functions is record
      --  int (*test)(void);
      test : int;
      --  ao_info* (*driver_info)(void);
      driver_info : ao_info_ptr;
      --  int (*device_init)(ao_device *device);
      device_init : int;
      --  int (*set_option)(ao_device *device, const char *key,
      --                  const char *value);
      set_option : int;
      --  int (*open)(ao_device *device, ao_sample_format *format);
      open : int;
      --  int (*play)(ao_device *device, const char *output_samples,
      --                   uint_32 num_bytes);
      play : int;
      --  int (*close)(ao_device *device);
      close : int;
      --  void (*device_clear)(ao_device *device);
      device_clear : void;
      --  char* (*file_extension)(void);
      file_extension : chars_ptr;
   end record;

   type ao_functions_ptr is access ao_functions;

   type ao_device is record
      ao_type : int; --  /* live output or file output? */
      driver_id : int;
      funcs : ao_functions_ptr;
      file : Interfaces.C_Streams.FILEs;
      client_byte_format : int;
      machine_byte_format : int;
      driver_byte_format : int;
      swap_buffer : chars_ptr;
      swap_buffer_size : int; --  /* Bytes allocated to swap_buffer */
      internal : void_ptr; --  /* Pointer to driver-specific data */
   end record;
   for ao_device'Size use 320;   --  C uses 40 bytes
   type ao_device_ptr is access all ao_device;


   type ao_sample_format is record
        bits : int := 16;         --  bits per sample
        rate : int := 44100;      --  samples per second (in a single channel)
        channels : int := 2;      --  number of audio channels
        byte_format : AO_Fmt := AO_Fmt_Native;
   end record;
   type ao_sample_format_ptr is access ao_sample_format;
   subtype ao_sample_fmt is ao_sample_format_ptr;

   type ao_option;
   type ao_option_ptr is access ao_option;

   type ao_option is record
      key : chars_ptr;
      value : chars_ptr;
      next : ao_option_ptr;
   end record;

   --
   --  C-Functions
   --    try to use the Ada-functions below instead (if possible)
   --

   procedure ao_initialize;
   pragma Import (C, ao_initialize, "ao_initialize");

   procedure ao_shutdown;
   pragma Import (C, ao_shutdown, "ao_shutdown");

   function ao_open_live (driver_id : int; format : ao_sample_format_ptr;
                          option : ao_option_ptr) return ao_device_ptr;
   pragma Import (C, ao_open_live, "ao_open_live");

   function ao_play (device : ao_device_ptr; output_samples : chars_ptr;
                     --  output_samples : sample_buffer_ptr;
                     num_bytes : Unsigned_32) return int;
   pragma Import (C, ao_play, "ao_play");

   function ao_close (device : ao_device_ptr) return int;
   pragma Import (C, ao_close, "ao_close");

   function ao_default_driver_id return int;
   pragma Import (C, ao_default_driver_id, "ao_default_driver_id");

   procedure ao_free_options (options : ao_option_ptr);
   pragma Import (C, ao_free_options, "ao_free_options");

   function ao_is_big_endian return int;
   pragma Import (C, ao_is_big_endian, "ao_is_big_endian");

   --
   --  Ada functions, all having 'a'(da) prepended to the C-function names
   --

   --  Plays the whole buffer
   procedure aao_play (device : ao_device_ptr; buffer : stereo_buffer_ptr);

   --  Plays only the first 'lastone' *samples* from buffer
   procedure aao_play (device : ao_device_ptr;
                       buffer : stereo_buffer_ptr; lastone : Positive);

   --  Close the device
   procedure aao_close (device : ao_device_ptr);

   --  Check for BigEndian architecture
   function aao_is_big_endian return Boolean;

end ao;
